/*
 *  Implementation of COSS Relationship Service for MICO
 *  Copyright (C) 1998 Karel Gardas with the assistance of Kay Roemer
 *                                                       & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 *  or to my private e-mail:
 *                 gardask@alpha.inf.upol.cz
 */


#include <string.h>

#include <mico/CosRelationships.h>

#ifndef __RelationshipFactory_impl_h__
#define __RelationshipFactory_impl_h__


//#define DEBUG 1


extern int counter;

class Relationship_impl;

template <class T>
class RelationshipFactory_impl : 
public virtual CosRelationships::RelationshipFactory_skel
{
  CORBA::UShort degree_of_rel;
  CosRelationships::RelationshipFactory::NamedRoleTypes* n_role_types;
  CORBA::Boolean first;
  CORBA::InterfaceDef* rel_type;
  char* name_of_relship_server;
public:
  RelationshipFactory_impl ();
  RelationshipFactory_impl 
    (CORBA::UShort d, 
     CosRelationships::RelationshipFactory::NamedRoleTypes* nrt);
  RelationshipFactory_impl 
    (const CORBA::BOA::ReferenceData &refdata, 
     CORBA::UShort d, 
     CosRelationships::RelationshipFactory::NamedRoleTypes* nrt,
     char* name);
  
  
  virtual CORBA::InterfaceDef_ptr relationship_type ();
  virtual CORBA::UShort degree ();
  virtual CosRelationships::RelationshipFactory::NamedRoleTypes* 
    named_role_types ();
  virtual CosRelationships::Relationship_ptr 
    create (const CosRelationships::NamedRoles& named_roles);
  virtual Relationship_impl* 
    create_relationship (const CosRelationships::NamedRoles& named_roles);
  virtual void destroy ();

  CORBA::ImplementationDef_ptr find_impl (const char *name);

};


template <class T>
RelationshipFactory_impl<T>::RelationshipFactory_impl ()
{
  degree_of_rel = 0;
  n_role_types = new CosRelationships::RelationshipFactory::NamedRoleTypes;
  n_role_types->length (0);
  first = TRUE;
  rel_type = CORBA::InterfaceDef::_nil ();
}


template <class T>
RelationshipFactory_impl<T>::RelationshipFactory_impl 
(const CORBA::BOA::ReferenceData &refdata,
 CORBA::UShort d,
 CosRelationships::RelationshipFactory::NamedRoleTypes* nrt,
 char* name)
  : CosRelationships::RelationshipFactory_skel (refdata)
{
  first = TRUE;
  degree_of_rel = d;
  /*
  int length = nrt->length ();
  n_role_types->length (length);
#if DEBUG
  cout << "  <RelationshipFactory_impl> constructor...\n";
  cout << "  <RelationshipFactory_impl> NamedRoleTypes length (): "
       << length << "\n";
  for (CORBA::ULong i=0; i<nrt.length; i++) {
    (*n_role_types)[i].name = CORBA::string_dup ((*nrt)[i].name);
    (*n_role_types)[i].
    */
  n_role_types = nrt;
  name_of_relship_server = CORBA::string_dup (name);
}


template <class T>
RelationshipFactory_impl<T>::RelationshipFactory_impl 
(CORBA::UShort d,
 CosRelationships::RelationshipFactory::NamedRoleTypes* nrt)
{
  first = TRUE;
  degree_of_rel = d;
  n_role_types = nrt;
}




template <class T>
CORBA::InterfaceDef_ptr
RelationshipFactory_impl<T>::relationship_type ()
{
#if DEBUG
  cout << "  <RelationshipFactory_impl> relationship_type():\n";
#endif
  CORBA::BOA_var boa = _boa ();
  CORBA::ORB_ptr orb = _orb ();

  if (first) {
#if DEBUG
    cout << "  <RelationshipFactory_impl> first call...\n";
#endif
    CosRelationships::NamedRoles tmp_named_roles;
    //tmp_named_roles = new CosRelationships::NamedRoles;
    Relationship_impl* tmp_relationship
      = create_relationship (tmp_named_roles);
    CosRelationships::Relationship_ptr relationship;
    relationship = tmp_relationship;
    rel_type = relationship->_get_interface ();
    first = FALSE;
    CORBA::release (relationship);
  }
#if DEBUG
  cout << "  <RelationshipFactory_impl> destroy relationship\n";
  cout << "  <RelationshipFactory_impl> describe interface of relationship:\n";
  CORBA::InterfaceDef::FullInterfaceDescription_var full_of_relationship
      = rel_type ->describe_interface ();
  cout << "  <RelationshipFactory_impl> name: " << full_of_relationship->name 
       << "\n";
  cout << "  <RelationshipFactory_impl> id: " << full_of_relationship->id 
       << "\n";
  cout << "  <RelationshipFactory_impl> defined_in: " 
       << full_of_relationship->defined_in << "\n";
  cout << "  <RelationshipFactory_impl> version: "
       << full_of_relationship->version << "\n";
#endif

      return CORBA::InterfaceDef::_duplicate (rel_type);

}


template <class T>
CORBA::UShort
RelationshipFactory_impl<T>::degree ()
{
  return degree_of_rel;
}


template <class T>
CosRelationships::RelationshipFactory::NamedRoleTypes*
RelationshipFactory_impl<T>::named_role_types ()
{
  return n_role_types;
}





template <class T>
CosRelationships::Relationship_ptr
RelationshipFactory_impl<T>::create 
(const CosRelationships::NamedRoles& named_roles)
{
#if DEBUG
  cout << "  <RelationshipFactory_impl> create (...);\n";
#endif

  CORBA::BOA_var boa = _boa();
  CORBA::ORB_var orb = _orb();

  if (degree_of_rel != named_roles.length ()) {
#if DEBUG
    cout << "  <RelationshipFactory_impl> Degree ERROR!!!\n";
#endif
    CosRelationships::RelationshipFactory::DegreeError ex;
    ex.required_degree = degree_of_rel;
    mico_throw (ex);
  }

  CosRelationships::NamedRoles tmp_nrs;
  tmp_nrs.length (0);

#if DEBUG
  cout << "  <RelationshipFactory_impl> exist duplicate role name?\n";
#endif
  // exist duplicate role name?
  CORBA::Boolean is_duplicate = FALSE;
  CosRelationships::RoleName tmp_role_name;
  for (CORBA::ULong i=0; i<named_roles.length (); i++) {
    for (CORBA::ULong j=0; j<named_roles.length (); j++) {
      if (named_roles[i].name == named_roles[j].name)
	if (i != j) {
#if DEBUG
	  cout << "  <RelationshipFactory_impl> duplicate!\n";
#endif
	  tmp_nrs.length (tmp_nrs.length () + 1);
	  tmp_nrs[tmp_nrs.length () - 1] = named_roles[i];
	  is_duplicate = TRUE;
	  break;
	}
    }
    //if (is_duplicate)
    //  break;
  }  
  if (is_duplicate) {
#if DEBUG
    cout << "  <RelationshipFactory_impl> is duplicate!!!\n";
#endif
    CosRelationships::RelationshipFactory::DuplicateRoleName ex;
#if DEBUG
    cout << "  <RelationshipFactory_impl> ex.culprits = tmp_nrs;\n";
#endif
    ex.culprits = tmp_nrs;
#if DEBUG
    cout << "  <RelationshipFactory_impl> DuplicateRoleName Exception!!!\n";
#endif
    mico_throw (ex);
  }
#if DEBUG
  cout << "  <RelationshipFactory_impl> continue...\n";
#endif

  CORBA::Boolean is_in = FALSE;
  CORBA::Boolean UnknownRoleNameError = FALSE;
  
  tmp_nrs.length (0);
#if DEBUG
  cout << "  <RelationshipFactory_impl> comparing roles names.\n";
#endif
  //comparing roles names
  for (CORBA::ULong i=0; i<named_roles.length (); i++) {
    for (CORBA::ULong j=0; j<n_role_types->length (); j++) {
      if (named_roles[i].name == (*n_role_types)[j].name) {
	is_in = TRUE;
	break;
      }
    }
    if (!is_in) {
      tmp_nrs.length (tmp_nrs.length () + 1);
      tmp_nrs[tmp_nrs.length () - 1] = named_roles[i];
      UnknownRoleNameError = TRUE;
    }
    is_in = FALSE;
  }
  if (UnknownRoleNameError) {
#if DEBUG
    cout << "  <RelationshipFactory_impl> UnknownRoleName ERROR!!!\n";
#endif
    CosRelationships::RelationshipFactory::UnknownRoleName ex;
    ex.culprits = tmp_nrs;
    mico_throw (ex);
  }


  CORBA::Boolean RoleTypeError = FALSE;
  tmp_nrs.length (0);
  
  //comparing roles types
#if DEBUG
  cout << "  <RelationshipFactory_impl> comparing roles types.\n";
  cout << "  <RelationshipFactory_impl> n_role_types->length (): "
       << n_role_types->length () << "\n";
  cout << "  <RelationshipFactory_impl> named_roles.length (): " 
       << named_roles.length () << "\n";
#endif
  for (CORBA::ULong i=0; i<n_role_types->length (); i++) {
    for (CORBA::ULong j=0; j<named_roles.length (); j++) {
#if DEBUG
      cout << "  <RelationshipFactory_impl> i, j = " << i << ", " << j << "\n";
#endif
      if (strcmp((*n_role_types)[i].name, named_roles[j].name) == 0) {
#if DEBUG
	cout << "  <RelationshipFactory_impl> the same names...\n";
	if (CORBA::is_nil ((*n_role_types)[i].named_role_type))
	  cout << "  <RelationshipFactory_impl> named_role_type is NIL!\n";
	else
	  cout << "  <RelationshipFactory_impl> named_role_type is value\n";
#endif
	if (!(*n_role_types)[i].named_role_type->is_a 
	    (named_roles[j].aRole->_repoid ())) {
#if DEBUG
	  cout << "  <RelationshipFactory_impl> RoleTypeError!\n";
#endif
	  RoleTypeError = TRUE;
	  tmp_nrs.length (tmp_nrs.length () + 1);
	  tmp_nrs[tmp_nrs.length () - 1] = named_roles[j];
	}
	else {
#if DEBUG
	  cout << "  <RelationshipFactory_impl> RoleType OK!\n";
#endif
	}
	  
      }
    }
  }
  if (RoleTypeError) {
#if DEBUG
    cout << "  <RelationshipFactory_impl> RoleTypeError !!!\n";
#endif
    CosRelationships::RelationshipFactory::RoleTypeError ex;
    ex.culprits = tmp_nrs;
    mico_throw (ex);
  }

#if DEBUG
    cout << "  <RelationshipFactory_impl> create_relationship ...\n";
    for (CORBA::ULong i=0; i<named_roles.length (); i++)
      cout << i << ". " << named_roles[i].aRole->_repoid () << " - " 
	   << named_roles[i].name << "\n";
#endif

  Relationship_impl* rel_impl 
    = create_relationship (named_roles);

  CosRelationships::Relationship_ptr rel = rel_impl;
//  boa->deactivate_obj (rel);

#if DEBUG
    cout << "  <RelationshipFactory_impl> find_impl...\n";
#endif

  CORBA::ImplementationDef_var impl = find_impl (name_of_relship_server);

#if DEBUG
    cout << "  <RelationshipFactory_impl> change_implementation ...\n";
#endif

  boa->change_implementation (rel, impl);

  CORBA::String_var ref = orb->object_to_string (rel);
  CORBA::release (rel);
  CORBA::Object_var obj = orb->string_to_object (ref);
  rel = CosRelationships::Relationship::_narrow (obj);
 

  CosRelationships::RelationshipFactory::MaxCardinalityExceeded ex;
  ex.culprits.length (0);
  CosRelationships::Role::RelationshipTypeError ex2;
  int ex_index = 0;
  CosRelationships::RelationshipHandle rh;
  rh.constant_random_id = rel->constant_random_id ();
  rh.the_relationship = CosRelationships::Relationship::_duplicate (rel);

  CosRelationships::NamedRoles tmp_nr;
  tmp_nr.length (0);

  for (CORBA::ULong i=0; i<named_roles.length (); i++) { 
    try {
      named_roles[i].aRole
      ->link (rh, named_roles);
    }
    catch
      (CosRelationships::RelationshipFactory::MaxCardinalityExceeded_catch &e)
      {
	ex.culprits.length (ex.culprits.length () + 1);
	ex.culprits[ex_index].aRole 
	  = CosRelationships::Role::_duplicate (e->culprits[0].aRole);
	ex.culprits[ex_index].name = CORBA::string_dup (e->culprits[0].name);
	tmp_nr.length (tmp_nr.length () + 1);
	tmp_nr[tmp_nr.length () - 1].aRole 
	  = CosRelationships::Role::_duplicate (named_roles[i].aRole);
	tmp_nr[tmp_nr.length () - 1].name =
	    CORBA::string_dup (named_roles[i].name);
	ex_index++;
      }
    catch 
      (CosRelationships::Role::RelationshipTypeError_catch &e)
      {
	if (ex_index > 0) {
	  for (CORBA::ULong j=0; j<named_roles.length (); j++) {
	    is_in = FALSE;
	    for (CORBA::ULong k=0; k<ex.culprits.length (); k++) 
	      if (ex.culprits[k].aRole->_is_equivalent (named_roles[j].aRole)){
		is_in = TRUE;
		break;
	      }
	    if (!is_in) {
              try {
	        named_roles[j].aRole->unlink (rh);
	      } catch (CosRelationships::Role::UnknownRelationship_catch &e2) {
		cerr << "create: unknown relship!" << endl;
	      }
	    }
	  }
	}
	rel->destroy ();
	mico_throw (ex2);
      }
  	  
    //delete rh;
  }
  if (ex_index > 0) {
    for (CORBA::ULong i=0; i<named_roles.length (); i++) {
      is_in = FALSE;
      for (CORBA::ULong j=0; j<ex.culprits.length (); j++) 
	if (ex.culprits[j].aRole->_is_equivalent (named_roles[i].aRole)) {
	  is_in = TRUE;
	  break;
	}
      if (!is_in) {
	try {
	  named_roles[i].aRole->unlink (rh);
        } catch (CosRelationships::Role::UnknownRelationship_catch &e2) {
	  cerr << "create: unknown relship!" << endl;
        }
      }
    }
    rel->destroy (); 
    mico_throw (ex);
  }
  if (first) {
    rel_type = rel->_get_interface ();
    first = FALSE;
  }

  return CosRelationships::Relationship::_duplicate (rel);
}


template <class T>
Relationship_impl*
RelationshipFactory_impl<T>::create_relationship (
			      const CosRelationships::NamedRoles& named_roles)
{
#if DEBUG
  cout << "  <RelationshipFactory_impl> create_relationship (...);\n";
#endif
  return new T (named_roles);
}


template <class T>
void
RelationshipFactory_impl<T>::destroy ()
{
  CORBA::BOA_var boa = _boa ();
  CORBA::ORB_var orb = _orb ();
  counter--;
  if (counter == 0) {
#if DEBUG
    cout << "  <RelationshipFactory_impl> END!\n";
#endif 
    boa->deactivate_impl (CORBA::ImplementationDef::_nil ());
    orb->shutdown (TRUE);
  }
}


template <class T>
CORBA::ImplementationDef_ptr
RelationshipFactory_impl<T>::find_impl (const char *name)
{
#if DEBUG
  cout << "  <RelationshipFactory_impl> find_impl ...\n";
#endif
  CORBA::Object_var obj =
      _orbnc()->resolve_initial_references ("ImplementationRepository");
  assert (!CORBA::is_nil (obj));

  CORBA::ImplRepository_var imr = CORBA::ImplRepository::_narrow (obj);
  assert (!CORBA::is_nil (imr));

  CORBA::ImplRepository::ImplDefSeq_var impls = imr->find_by_name (name);
  assert (impls->length() > 0);
  return CORBA::ImplementationDef::_duplicate (impls[0]);
}



#endif

