// -*- c++ -*-
/*
 *  MICO --- a free CORBA implementation
 *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __dynany_impl_h__
#define __dynany_impl_h__


class DynAny_impl : virtual public CORBA::DynAny {
protected:
    vector<CORBA::DynAny_var> _elements;
    CORBA::Long _index;
    CORBA::TypeCode_var _type;

    static CORBA::ORB_ptr _factory();

    virtual void update_element (CORBA::Long idx);
public:
    DynAny_impl ();
    virtual ~DynAny_impl ();

    void assign (CORBA::DynAny_ptr dyn_any);
    CORBA::DynAny_ptr copy ();

    CORBA::TypeCode_ptr type ();
    virtual void destroy ();
    void insert_boolean (CORBA::Boolean value);
    void insert_octet (CORBA::Octet value);
    void insert_char (CORBA::Char value);
    void insert_wchar (CORBA::WChar value);
    void insert_short (CORBA::Short value);
    void insert_ushort (CORBA::UShort value);
    void insert_long (CORBA::Long value);
    void insert_longlong (CORBA::LongLong value);
    void insert_ulong (CORBA::ULong value);
    void insert_ulonglong (CORBA::ULongLong value);
    void insert_float (CORBA::Float value);
    void insert_double (CORBA::Double value);
    void insert_longdouble (CORBA::LongDouble value);
    void insert_string (const char* value);
    void insert_wstring (const CORBA::WChar* value);
    void insert_reference (CORBA::Object_ptr value);
    void insert_typecode (CORBA::TypeCode_ptr value);
    void insert_any (const CORBA::Any &value);
    void insert_valuetype (CORBA::ValueBase *value);

    CORBA::Boolean get_boolean ();
    CORBA::Octet get_octet ();
    CORBA::Char get_char ();
    CORBA::WChar get_wchar ();
    CORBA::Short get_short ();
    CORBA::UShort get_ushort ();
    CORBA::Long get_long ();
    CORBA::LongLong get_longlong ();
    CORBA::ULong get_ulong ();
    CORBA::ULongLong get_ulonglong ();
    CORBA::Float get_float ();
    CORBA::Double get_double ();
    CORBA::LongDouble get_longdouble ();
    char* get_string ();
    CORBA::WChar* get_wstring ();
    CORBA::Object_ptr get_reference ();
    CORBA::TypeCode_ptr get_typecode ();
    CORBA::Any* get_any ();
    CORBA::ValueBase* get_valuetype ();

    CORBA::DynAny_ptr current_component ();
    CORBA::Boolean next ();
    CORBA::Boolean seek (CORBA::Long index);
    void rewind ();

    virtual void from_any (const CORBA::Any& value) = 0;
    virtual CORBA::Any* to_any () = 0;
};


class DynBasic_impl : virtual public DynAny_impl,
		      virtual public CORBA::DynBasic {
private:
    CORBA::Any _value;

public:
    DynBasic_impl (CORBA::TypeCode_ptr);
    DynBasic_impl (const CORBA::Any &);
    virtual ~DynBasic_impl ();

    void destroy ();
    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();
};

class DynFixed_impl : virtual public DynAny_impl,
		      virtual public CORBA::DynFixed {
private:
    CORBA::Any _value;

public:
    DynFixed_impl (CORBA::TypeCode_ptr);
    DynFixed_impl (const CORBA::Any &);
    virtual ~DynFixed_impl ();

    void destroy ();
    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    CORBA::OctetSeq* get_value ();
    void set_value (const CORBA::OctetSeq& val);
};

class DynEnum_impl : virtual public DynAny_impl,
		     virtual public CORBA::DynEnum {
private:
    CORBA::Any _value;

public:
    DynEnum_impl (CORBA::TypeCode_ptr);
    DynEnum_impl (const CORBA::Any &);
    virtual ~DynEnum_impl ();

    void destroy ();
    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    char* value_as_string ();
    void value_as_string (const char* value);
    CORBA::ULong value_as_ulong ();
    void value_as_ulong (CORBA::ULong value);
};


class DynStruct_impl : virtual public DynAny_impl,
		       virtual public CORBA::DynStruct {
private:
    CORBA::Boolean _isexcept;
public:
    DynStruct_impl (CORBA::TypeCode_ptr);
    DynStruct_impl (const CORBA::Any &);
    virtual ~DynStruct_impl ();

    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    CORBA::FieldName current_member_name ();
    CORBA::TCKind current_member_kind ();
    CORBA::NameValuePairSeq* get_members ();
    void set_members (const CORBA::NameValuePairSeq& value);
};


class DynUnion_impl : virtual public DynAny_impl,
		      virtual public CORBA::DynUnion {
private:
    virtual void update_element (CORBA::Long idx);
    CORBA::Long _member_idx;
public:
    DynUnion_impl (CORBA::TypeCode_ptr);
    DynUnion_impl (const CORBA::Any &);
    virtual ~DynUnion_impl ();

    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    CORBA::Boolean set_as_default ();
    void set_as_default (CORBA::Boolean value);
    CORBA::FieldName member_name ();
    void member_name (const char *value);
    CORBA::DynAny_ptr discriminator ();
    CORBA::TCKind discriminator_kind ();
    CORBA::DynAny_ptr member ();
    CORBA::TCKind member_kind ();
};

class DynSequence_impl : virtual public DynAny_impl,
			 virtual public CORBA::DynSequence {
private:
    CORBA::ULong _length;
public:
    DynSequence_impl (CORBA::TypeCode_ptr);
    DynSequence_impl (const CORBA::Any &);
    virtual ~DynSequence_impl ();

    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    CORBA::ULong length ();
    void length (CORBA::ULong value);
    CORBA::AnySeq* get_elements ();
    void set_elements (const CORBA::AnySeq& value);
};


class DynArray_impl : virtual public DynAny_impl,
		      virtual public CORBA::DynArray {
private:
    CORBA::ULong _length;
public:
    DynArray_impl (CORBA::TypeCode_ptr);
    DynArray_impl (const CORBA::Any &);
    virtual ~DynArray_impl ();

    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();

    CORBA::AnySeq* get_elements ();
    void set_elements (const CORBA::AnySeq& value);
};

class DynValue_impl : virtual public DynAny_impl,
		      virtual public CORBA::DynValue {
private:
    CORBA::Boolean _isvalue;
public:
    DynValue_impl (const CORBA::Any &);
    virtual ~DynValue_impl ();

    void from_any (const CORBA::Any& value);
    CORBA::Any* to_any ();
    CORBA::Boolean next ();
    CORBA::Boolean seek (CORBA::Long index);
    void rewind ();

    CORBA::FieldName current_member_name ();
    CORBA::TCKind current_member_kind ();
    CORBA::Visibility current_member_visibility ();
    CORBA::NameValueAccessSeq* get_members ();
    void set_members (const CORBA::NameValueAccessSeq& value);
};

#endif
